<?php

namespace Drupal\entity_reference_media\Plugin\Field\FieldFormatter;

use Drupal\Core\Field\FieldItemListInterface;
use Drupal\Core\Field\Plugin\Field\FieldFormatter\EntityReferenceEntityFormatter;
use Drupal\Core\Form\FormStateInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Plugin implementation of the 'entity_reference_media_entity' formatter.
 *
 * @FieldFormatter(
 *   id = "entity_reference_media_entity",
 *   label = @Translation("Rendered entity"),
 *   description = @Translation("Display the referenced entities rendered by entity_view(), with optional field overrides."),
 *   field_types = {
 *     "entity_reference_media"
 *   }
 * )
 */
class EntityReferenceMediaFormatter extends EntityReferenceEntityFormatter {

  /**
   * The EntityField Manager service.
   */
  protected $entityFieldManager;

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    $instance = parent::create($container, $configuration, $plugin_id, $plugin_definition);
    $instance->entityFieldManager = $container->get('entity_field.manager');
    return $instance;
  }


  /**
   * {@inheritdoc}
   */
  public static function defaultSettings() {
    return [
        'caption_action' => 'title',
      ] + parent::defaultSettings();
  }

  /**
   * {@inheritdoc}
   */
  public function settingsForm(array $form, FormStateInterface $form_state) {
    $target_entity_type = $this->getFieldSetting('target_type');

    $fieldList = array_merge_recursive(
      $this->entityFieldManager->getFieldMapByFieldType('string_long')[$target_entity_type],
      $this->entityFieldManager->getFieldMapByFieldType('string')[$target_entity_type],
    );

    $fieldOptions = [];
    foreach ($fieldList as $key => $field) {
      $fieldOptions[$key] = $key;
    }

    $elements = parent::settingsForm($form, $form_state);
    $elements['caption_action'] = [
      '#type' => 'select',
      '#options' => [
          'title' => t('Entity title'),
        ] + $fieldOptions,
      '#title' => t('Use custom caption to override'),
      '#default_value' => $this->getSetting('caption_action'),
      '#required' => TRUE,
    ];

    return $elements;
  }

  /**
   * {@inheritdoc}
   */
  public function settingsSummary() {
    $summary = parent::settingsSummary();

    $caption_action = $this->getSetting('caption_action');
    $summary[] = t('Per-entity @override override', ['@override' => $caption_action]);

    return $summary;
  }

  /**
   * {@inheritdoc}
   */
  public function viewElements(FieldItemListInterface $items, $langcode) {
    $view_mode = $this->getSetting('view_mode');
    $elements = [];

    foreach ($this->getEntitiesToView($items, $langcode) as $delta => $entity) {
      $clone = clone $entity;

      if (!empty($items[$delta]->custom_caption) && !$items[$delta]->default_caption) {
        $override = $this->getSetting('caption_action');
        switch ($override) {
          case 'title':
            $title_key = $clone->getEntityType()->getKey('label');
            $title_key = $title_key ?: 'title';
            $clone->$title_key = $items[$delta]->override;
            break;

          default:
            $clone->set($override, [
              'value' => $items[$delta]->custom_caption,
              'format' => $items[$delta]->override_format,
            ]);
            break;
        }
      }

      $view_builder = $this->entityTypeManager->getViewBuilder($entity->getEntityTypeId());
      $elements[$delta] = $view_builder->view($clone, $view_mode, $entity->language()
        ->getId());
    }

    return $elements;
  }

}
